﻿using System;
using System.ComponentModel;
using System.Data;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Intelligence;
using Smart;
using Package;
using System.Windows.Forms.DataVisualization.Charting;
using System.Media;
using System.IO;
using NDde.Client;

namespace TradeCenter
{

    public enum ORDER_STATUS
    {
        ORDER_UNDECIDED = 0,
        ORDER_PENDING = 1,
        ORDER_CLOSED = 2,
        ORDER_DELETED = 3,
    }


    delegate void SetAddInfoCallBack(string text);


    public partial class Form1 : Form
    {

        /// MSCI Taiwan DDE

        DdeClient dcBid;

        string strMT4Service = "MT4";
        string strBIDTopic = "BID";
        string strMsciSymbol = "MSCI_TAIWAN";
        bool bDDEConnected = false;


        ///// Quote parameters

        string strID = "";
        string strPwd = "";
        string tbTAccount = "";
        string brokerID = "";
        string subAccount = "";


        bool bSound = true;
        decimal dSpeedLimit = 10;

        string strSymbolIni = @"symbol.ini";
        string strAccountIni = @"account.ini";


        /// Trade parameters
        string symbolID = "TXFA5";
        int iInitialQty = 1;
        int iQtyLimit = 500;

        bool bSlippage = true;
        decimal dPriceStep = 1;

        int iPositionIndex = 3; //// 3 - Automatic order position 
        string strOrderServerHost = "itrade.kgi.com.tw";
        string strOrderServerPort = "443";
        string strQuoteServerHost = "iquote.kgi.com.tw";
        ushort strQuotePort = 8000;
        
        /// 

        // Charting parameters

        bool bGraphicsInitialized = false;
        double dRange = 0.005;  /// 0.5% y-axis range
        double yMax = 0.0;
        double yMin = 0.0;
        int xx = 0;
        double yBoundary = 1.0;
        double yBoxMax = 0.0, yBoxMin = 0.0;
        double dLadder = 0.0, dLadderPrev = 0.0;
        double dLadderSize = 4.0;
        bool bLadderBottom = true;

      

        /// Quote
        Intelligence.QuoteCom quoteCom;
        Dictionary<string, int> RecoverMap = new Dictionary<string, int>();
        private UTF8Encoding encoding = new System.Text.UTF8Encoding();


        /// 

        decimal dPricePrev = 0, dPriceCurrent = 0;
        bool bTimerPricespeedcalled = true;

       


        TaiFexCom tfcom;

        SoundPlayer playAlert = new System.Media.SoundPlayer("alert.wav");
        SoundPlayer playSignal = new System.Media.SoundPlayer("signal.wav");

        bool bBuy = false;
        bool bSell = false;

       
        bool bAutoLogin = false;
        bool bAutoLogoff = false;

        

        public Form1()
        {
            InitializeComponent();

            SymbolInitialized();


            lbAskVol.Text = "";
            lbBidVol.Text = "";

            this.buylabel.Text = "";
            this.selllabel.Text = "";
            this.labelServerTime.Text = "";
            this.lbSpeed.Text = "漲跌速: -";


            this.SellBtn.BackColor = System.Drawing.Color.LimeGreen;
            this.SellBtn.ForeColor = System.Drawing.SystemColors.ButtonHighlight;


            this.BuyBtn.BackColor = System.Drawing.Color.Tomato;
            this.BuyBtn.ForeColor = System.Drawing.SystemColors.ButtonHighlight;


            this.kgiTimer.Enabled = true;
            this.kgiTimer.Tick += new EventHandler(OnCalculatePriceSpeed);
            this.kgiTimer.Start();

            ////  Chart default = Invisible 
            futureschart.Visible = false;
            tabControlKGITwn.Visible = false;
            this.Width = 320;
            this.CenterToScreen();


            this.Text = "KGI Futures";

            this.SymbolBox.Text = symbolID;
            this.qtyBox.Text = iInitialQty.ToString();

            mscitaiwanLbMscitaiwanprice.Text = "";
            msciTaiwanLbSpreadPrice.Text = "";
            mscitaiwanLbTaiexprice.Text = "";

            

        }

        private void SymbolInitialized()
        {
            string[] strAccountSplit;

            if (!File.Exists(strSymbolIni))
            {
                System.IO.File.WriteAllText(strSymbolIni, symbolID);
            }
            else
            {
                symbolID = System.IO.File.ReadAllText(strSymbolIni);
            }

            if (!File.Exists(strAccountIni))
            {
                AddInfo("The account.ini file doesn't exist."
                    + Environment.NewLine
                    + "Please create this file with account and password info.");
            }
            else
            {
                string strIdPwd = "";
                strIdPwd = System.IO.File.ReadAllText(strAccountIni);

                if (string.IsNullOrEmpty(strIdPwd))
                {
                    AddInfo("KGI account ID & password are not stored at account.ini file."
                   + Environment.NewLine
                   + "Please add those info at that file and restart the program.");
                }
                else
                {
                    strIdPwd = strIdPwd.Trim();

                    strAccountSplit = strIdPwd.Split(',');

                    brokerID = strAccountSplit[0];
                    tbTAccount = strAccountSplit[1];
                    strID = strAccountSplit[2];
                    strPwd = strAccountSplit[3];
                }
            }
        }

        private void OnCalculatePriceSpeed(Object myObject, EventArgs myEventArgs)
        {

            StringBuilder sb = new StringBuilder();
            
            if (DateTime.Now.Hour == 8 && (DateTime.Now.Minute >= 40 && DateTime.Now.Minute < 45)
               && bAutoLogin)
            {
                bAutoLogin = false;
                bAutoLogoff = true;
                this.kgiTimer.Stop();
                
                Login();

                //mscitaiwanCbEnabled.Checked = true;
                
                this.kgiTimer.Start();
            }

            

            if (DateTime.Now.Hour == 13 && DateTime.Now.Minute == 50 && bAutoLogoff)
            {
                bAutoLogin = true;
                bAutoLogoff = false;

                this.kgiTimer.Stop();

                Logoff();

                this.kgiTimer.Start();
            }

            decimal dPriceDelta;

            if (!string.IsNullOrEmpty(buylabel.Text.Trim()))
            {
                dPriceCurrent = decimal.Parse(buylabel.Text.Trim());
            }
            else
            {
                return;
            }

            if (bTimerPricespeedcalled && dPriceCurrent != 0)
            {
                dPricePrev = dPriceCurrent;
                bTimerPricespeedcalled = false;
            }

            dPriceDelta = dPriceCurrent - dPricePrev;

            lbSpeed.Text = "漲跌速:  " + dPriceDelta.ToString();


            if (Math.Abs(dPriceDelta) > dSpeedLimit)
            {
                AddInfo("Alert: " + dPriceDelta.ToString() + " " + dPriceCurrent.ToString() + " " + this.labelServerTime.Text);

                if (bSound)
                {
                    playAlert.Play();
                }
            }

            dPricePrev = dPriceCurrent;



        }



        #region TaiFexCom API 事件

        private void OnRcvServerTime(Object sender, DateTime serverTime, int ConnQuality)
        {
            if (this.InvokeRequired)
            {
                Smart.OnRcvServerTime_EventHandler d = new Smart.OnRcvServerTime_EventHandler(OnRcvServerTime);
                this.Invoke(d, new object[] { sender, serverTime, ConnQuality });
                return;
            }

            labelServerTime.Text = String.Format("{0:yyyy/MM/dd HH:mm:ss}", serverTime);




        }

        private void OnRcvMessage(object sender, PackageBase package)
        {


            if (this.InvokeRequired)
            {
                Smart.OnRcvMessage_EventHandler d = new Smart.OnRcvMessage_EventHandler(OnRcvMessage);
                this.Invoke(d, new object[] { sender, package });
                return;
            }
            //string msg = "";
            int cnt = 0;

            StringBuilder sb;
            switch ((DT)package.DT)
            {
                case DT.LOGIN:              //登入
                    P001503 p001503 = (P001503)package;
                    if (p001503.Code != 0)
                    {
                        AddInfo("凱基下單登入失敗 CODE=" + p001503.Code + " " + tfcom.GetMessageMap(p001503.Code));

                    }
                    else
                    {
                        //TxtLayout.Text = "交易版面:" + Convert.ToString(p001503.Layout) + "可註冊檔數:" + p001503.Qnum + "  " + p001503.LoginType;
                        //AddInfo("交易版面:" + Convert.ToString(p001503.Layout) + "可註冊檔數:" + p001503.Qnum + "  " + p001503.LoginType);

                    }
                    break;
                case DT.FINANCIAL:      //權益數
                    P001605 p001605 = (P001605)package;
                    //AddInfo("1605 筆數" + p001605.Count);
                    sb = new StringBuilder();
                    if (p001605.Count > 0)
                    {
                        foreach (P001605_2 p001605_b in p001605.p001605_2)
                        {
                            sb.Append("----------------第" + (++cnt)).AppendLine("筆-------------------")
                            .Append("分公司代號").Append(p001605_b.BrokerId).Append('\t').AppendLine("帳號").AppendLine(p001605_b.Account)
                            .Append("幣別=").AppendLine(p001605_b.CURRENCY)
                            .Append("前日帳款餘額=").AppendLine(p001605_b.LCTDAB)
                            .Append("手續費=").AppendLine(p001605_b.ORIGNFEE)
                            .Append("匯率=").AppendLine(p001605_b.TAXAMT)
                            .Append("期交稅=").AppendLine(p001605_b.CTAXAMT)
                            .Append("存提金額=").AppendLine(p001605_b.DWAMT)
                            .Append("平倉損益=").AppendLine(p001605_b.OSPRTLOS)
                            .Append("未平倉損益=").AppendLine(p001605_b.PRTLOS)
                            .Append("買方選擇權市值=").AppendLine(p001605_b.BMKTVAL)
                            .Append("賣方選擇權市值=").AppendLine(p001605_b.SMKTVAL)
                            .Append("下單預扣權利金=").AppendLine(p001605_b.OPREMIUM)
                            .Append("當日權利金收支=").AppendLine(p001605_b.TPREMIUM)
                            .Append("淨值=").AppendLine(p001605_b.EQUITY)
                            .Append("原始保證金=").AppendLine(p001605_b.IAMT)
                            .Append("維持保證金=").AppendLine(p001605_b.MAMT)
                            .Append("可用餘額=").AppendLine(p001605_b.EXCESS)
                            .Append("下單可用保證金=").AppendLine(p001605_b.ORDEXCESS)
                            .Append("當日委託保證金=").AppendLine(p001605_b.ORDAMT)
                            .Append("履約損益=").AppendLine(p001605_b.ExProfit)
                            .Append("變動權利金=").AppendLine(p001605_b.Premium)
                            .Append("洗價時間=").AppendLine(p001605_b.PTime)
                            .Append("盤中浮動損益=").AppendLine(p001605_b.FloatProfit)
                            .Append("昨日未平倉損益=").AppendLine(p001605_b.LASSPRTLOS)
                            .Append("到期結算保證金=").AppendLine(p001605_b.CLOSEAMT)
                            .Append("足額原始保證金=").AppendLine(p001605_b.ORDIAMT)
                            .Append("足額維持保證金=").AppendLine(p001605_b.ORDMAMT)
                            .Append("當沖原始保證金=").AppendLine(p001605_b.DayTradeAMT)
                            .Append("多空減收保證金=").AppendLine(p001605_b.ReductionAMT)
                            .Append("當沖應補保證金=").AppendLine(p001605_b.CreditAMT)
                            .Append("本日餘額=").AppendLine(p001605_b.balance)
                            .Append("本日權利金收入=").AppendLine(p001605_b.IPremium)
                            .Append("本日權利金支出=").AppendLine(p001605_b.OPremium)
                            .Append("有價証券價值=").AppendLine(p001605_b.Securities)
                            .Append("有價証券抵繳金額=").AppendLine(p001605_b.SecuritiesOffset)
                            .Append("委託抵繳保證金=").AppendLine(p001605_b.OffsetAMT)
                            .Append("剩餘可抵繳金額=").AppendLine(p001605_b.Offset)
                            .Append("足額總市值風險=").AppendLine(p001605_b.FULLMTRISK)
                            .Append("足額風險係數 =").AppendLine(p001605_b.FULLRISK);
                            //AddInfo(sb.ToString());
                            sb.Remove(0, sb.Length);
                        }
                    }
                    break;
                case DT.INVENTORY_YESTERDAY:
                    P001601 pt01601 = (P001601)package;
                    //AddInfo(pt01601.ToLog());
                    break;
                case DT.INVENTORY_YESTERDAY_Detail:
                    P001622 pt01622 = (P001622)package;
                    //AddInfo(pt01622.ToLog());
                    break;
                case DT.INVENTORY:          //未平倉彙總
                    P001607 pt01607 = (P001607)package;
                    //AddInfo(pt01607.ToLog());
                    break;
                case DT.INVENTORY_DETAIL:   //未平倉明細
                    P001609 pt01609 = (P001609)package;
                    //AddInfo("[1609] CODE:" + pt01609.Code + "  Total Count:" + pt01609.Count);
                    StringBuilder sbLog = new StringBuilder();
                    int i = 0;
                    foreach (P001609_2 pb01609_2 in pt01609.p001609_2)
                    {
                        sbLog.Append("----------------第" + (++i)).Append("筆-------------------").Append(Environment.NewLine);
                        sbLog.Append("分公司代號=").Append(pb01609_2.BrokerId).Append("\t").Append("帳號=").AppendLine(pb01609_2.Account);
                        sbLog.Append("商品代碼=").Append(pb01609_2.ComID).Append("\t")
                            .Append("商品年月=").Append(pb01609_2.ComYM).Append("\t")
                            .Append("履約價=").AppendLine(pb01609_2.StrikePrice);
                        sbLog.Append("委託書號=").Append(pb01609_2.SEQNO).Append("\t").Append("委託書號1=").AppendLine(pb01609_2.OrdNo);
                        sbLog.Append("成交序號1=").Append(pb01609_2.MarketNo).Append(Environment.NewLine);
                        sbLog.Append("拆單序號1=").Append(pb01609_2.sNo).Append(Environment.NewLine);
                        sbLog.Append("成交日期=").Append(pb01609_2.TradeDate).Append(Environment.NewLine);
                        sbLog.Append("BS=").Append(pb01609_2.BS).Append(Environment.NewLine);
                        sbLog.Append("F:期貨 O:期權=").Append(pb01609_2.ComType).Append(Environment.NewLine);
                        sbLog.Append("未平倉量=").Append(pb01609_2.Qty).Append(Environment.NewLine);
                        sbLog.Append("結算價=").Append(pb01609_2.TrdPrice).Append(Environment.NewLine);
                        sbLog.Append("即時價=").Append(pb01609_2.MPrice).Append(Environment.NewLine);
                        sbLog.Append("未平倉損益=").Append(pb01609_2.PRTLOS).Append(Environment.NewLine);
                        sbLog.Append("原始保證金=").Append(pb01609_2.InitialMargin).Append(Environment.NewLine);
                        sbLog.Append("維持保證金=").Append(pb01609_2.MTMargin).Append(Environment.NewLine);
                        sbLog.Append("幣別=").Append(pb01609_2.Currency).Append(Environment.NewLine);
                        sbLog.Append("成交價=").Append(pb01609_2.DealPrice).Append(Environment.NewLine);
                        sbLog.Append("當日沖銷=").Append(pb01609_2.DayTrade).Append(Environment.NewLine);
                        sbLog.Append("SPREAD單 (Y/N) =").Append(pb01609_2.SPREAD).Append(Environment.NewLine);
                        if (pb01609_2.SPREAD == 'Y')
                        {
                            sbLog.Append("拆單序號2=").Append(pb01609_2.sNo2).Append(Environment.NewLine);
                            sbLog.Append("成交日期2=").Append(pb01609_2.TradeDate2).Append(Environment.NewLine);
                            sbLog.Append("商品代碼2=").Append(pb01609_2.ComID2).Append("\t")
                                .Append("商品年月2=").Append(pb01609_2.ComYM2).Append("\t")
                                .Append("履約價2=").AppendLine(pb01609_2.StrikePrice2);
                            sbLog.Append("BS2=").Append(pb01609_2.BS2).Append(Environment.NewLine);
                            sbLog.Append("F:期貨 O:期權=").Append(pb01609_2.ComType2).Append(Environment.NewLine);
                            sbLog.Append("未平倉量2=").Append(pb01609_2.Qty2).Append(Environment.NewLine);
                            sbLog.Append("結算價2=").Append(pb01609_2.TrdPrice2).Append(Environment.NewLine);
                            sbLog.Append("即時價2=").Append(pb01609_2.MPrice2).Append(Environment.NewLine);
                            sbLog.Append("未平倉損益2=").Append(pb01609_2.PRTLOS2).Append(Environment.NewLine);
                            sbLog.Append("原始保證金2=").Append(pb01609_2.InitialMargin2).Append(Environment.NewLine);
                            sbLog.Append("維持保證金2=").Append(pb01609_2.MTMargin2).Append(Environment.NewLine);
                            sbLog.Append("幣別2=").Append(pb01609_2.Currency2).Append(Environment.NewLine);
                            sbLog.Append("成交價2=").Append(pb01609_2.DealPrice2).Append(Environment.NewLine);
                        }
                        //AddInfo(sbLog.ToString());
                        sbLog.Remove(0, sbLog.Length);
                    }
                    break;
                case DT.FINANCIAL_COVER:          //平倉彙總
                    P001612 pt01612 = (P001612)package;
                    //AddInfo(pt01612.ToLog());
                    break;

                #region 分帳
                case DT.FINANCIAL_COVER_TRADER:      //平倉查詢 分帳
                    P001614 p001614 = (P001614)package;
                    AddInfo("1614 筆數" + p001614.Rows);
                    sb = new StringBuilder();
                    if (p001614.Rows > 0)
                    {
                        foreach (P001614_2 p001614_b in p001614.p001614_2)
                        {
                            sb.Append("----------------第" + (++cnt)).AppendLine("筆-------------------")
                            .Append("分公司代號=").Append(p001614_b.BrokerId).Append('\t').AppendLine("帳號=").AppendLine(p001614_b.Account)
                            .Append("Group=").AppendLine(p001614_b.Group)
                            .Append("交易員=").AppendLine(p001614_b.Trader)
                            .Append("交易所=").AppendLine(p001614_b.Exchange)
                            .Append("商品代碼=").AppendLine(p001614_b.ComID)
                            .Append("年月=").AppendLine(p001614_b.ComYM)
                            .Append("履約價=").AppendLine(p001614_b.StrikePrice)
                            .Append("CP=").AppendLine(p001614_b.CP)
                            .Append("幣別=").AppendLine(p001614_b.CURRENCY)
                            .Append("平倉損益==").AppendLine(p001614_b.PRTLOS)
                            .Append("交易稅=").AppendLine(p001614_b.CTAXAMT)
                            .Append("手費費").AppendLine(p001614_b.ORIGNFEE)
                            .Append("淨損益=").AppendLine(p001614_b.OSPRTLOS)
                            .Append("平倉口數=").AppendLine(p001614_b.Qty)
                            .Append("淨損益(TW)=").AppendLine(p001614_b.OSPRTLOS_TWD);
                            AddInfo(sb.ToString());
                            sb.Remove(0, sb.Length);
                        }
                    }
                    break;
                case DT.INVENTORY_DETAIL_TRADER:     // 庫存明細  分帳
                    P001618 pt01618 = (P001618)package;
                    sbLog = new StringBuilder();
                    AddInfo("[1618] CODE:" + pt01618.Code + "  Total Count:" + pt01618.Rows);
                    i = 0;
                    foreach (P001618_2 pt01618_2 in pt01618.p001618_2)
                    {
                        sbLog.Append("----------------第" + (++i)).Append("筆-------------------").Append(Environment.NewLine);

                        sbLog.Append("分公司代號=").Append(pt01618_2.BrokerId).Append("\t").Append("帳號=").AppendLine(pt01618_2.Account)
                                .Append("GROUP=").Append(pt01618_2.Group).Append("\t").Append("交易員=").Append(pt01618_2.Trader)
                                .Append("交易所=").Append(pt01618_2.Exchange).Append("\t").Append("場內編號").Append(pt01618_2.SeqNo)
                                .Append("上手代碼=").Append(pt01618_2.FcmActNo).Append("\t").Append("交易方式=").Append(pt01618_2.TradeType)
                                .Append("下單經紀商=").Append(pt01618_2.FCM).Append("\t").AppendLine("交割日期=").Append(pt01618_2.DeliveryDate)
                                .Append("結算日期=").Append(pt01618_2.CloseDate).Append("\t").AppendLine("下單方式=").Append(pt01618_2.WEB);

                        sbLog.Append("商品代碼=").Append(pt01618_2.ComID).Append("\t")
                            .Append("商品年月=").Append(pt01618_2.ComYM).Append("\t")
                            .Append("履約價=").AppendLine(pt01618_2.StrikePrice);
                        sbLog.Append("電子單號=").Append(pt01618_2.Cnt).Append("\t").Append("委託書號1=").AppendLine(pt01618_2.OrdNo);
                        sbLog.Append("成交序號1=").Append(pt01618_2.MarketNo).Append(Environment.NewLine);
                        sbLog.Append("拆單序號1=").Append(pt01618_2.sNo).Append(Environment.NewLine);
                        sbLog.Append("成交日期=").Append(pt01618_2.TradeDate).Append(Environment.NewLine);
                        sbLog.Append("BS=").Append(pt01618_2.BS).Append(Environment.NewLine);
                        sbLog.Append("F:期貨 O:期權=").Append(pt01618_2.ComType).Append(Environment.NewLine);
                        sbLog.Append("未平倉量=").Append(pt01618_2.Qty).Append(Environment.NewLine);
                        sbLog.Append("結算價=").Append(pt01618_2.TrdPrice).Append(Environment.NewLine);
                        sbLog.Append("即時價=").Append(pt01618_2.MPrice).Append(Environment.NewLine);
                        sbLog.Append("未平倉損益=").Append(pt01618_2.PRTLOS).Append(Environment.NewLine);
                        sbLog.Append("原始保證金=").Append(pt01618_2.InitialMargin).Append(Environment.NewLine);
                        sbLog.Append("維持保證金=").Append(pt01618_2.MTMargin).Append(Environment.NewLine);
                        sbLog.Append("幣別=").Append(pt01618_2.Currency).Append(Environment.NewLine);
                        sbLog.Append("成交價=").Append(pt01618_2.DealPrice).Append(Environment.NewLine);
                        sbLog.Append("當日沖銷=").Append(pt01618_2.DayTrade).Append(Environment.NewLine);
                        sbLog.Append("SPREAD單 (Y/N) =").Append(pt01618_2.SPREAD).Append(Environment.NewLine);
                        if (pt01618_2.SPREAD == 'Y')
                        {
                            sbLog.Append("拆單序號2=").Append(pt01618_2.sNo2).Append(Environment.NewLine);
                            sbLog.Append("成交日期2=").Append(pt01618_2.TradeDate2).Append("\t");
                            sbLog.Append("商品代碼2=").Append(pt01618_2.ComID2).Append("\t")
                                .Append("商品年月2=").Append(pt01618_2.ComYM2).Append("\t")
                                .Append("履約價2=").AppendLine(pt01618_2.StrikePrice2);
                            sbLog.Append("BS2=").Append(pt01618_2.BS2).Append(Environment.NewLine);
                            sbLog.Append("F:期貨 O:期權=").Append(pt01618_2.ComType2).Append(Environment.NewLine);
                            sbLog.Append("未平倉量2=").Append(pt01618_2.Qty2).Append(Environment.NewLine);
                            sbLog.Append("結算價2=").Append(pt01618_2.TrdPrice2).Append(Environment.NewLine);
                            sbLog.Append("即時價2=").Append(pt01618_2.MPrice2).Append(Environment.NewLine);
                            sbLog.Append("未平倉損益2=").Append(pt01618_2.PRTLOS2).Append(Environment.NewLine);
                            sbLog.Append("原始保證金2=").Append(pt01618_2.InitialMargin2).Append(Environment.NewLine);
                            sbLog.Append("維持保證金2=").Append(pt01618_2.MTMargin2).Append(Environment.NewLine);
                            sbLog.Append("幣別2=").Append(pt01618_2.Currency2).Append(Environment.NewLine);
                            sbLog.Append("成交價2=").Append(pt01618_2.DealPrice2).Append(Environment.NewLine);
                        }
                        AddInfo(sbLog.ToString());
                        sbLog.Remove(0, sbLog.Length);
                    }
                    break;
                case DT.INVENTORY_TRADER:   //庫存彙總   分帳
                    P001616 p001616 = (P001616)package;
                    AddInfo("1616 筆數" + p001616.Rows);
                    sb = new StringBuilder();
                    if (p001616.Rows > 0)
                    {
                        foreach (P001616_2 p001616_b in p001616.p001616_2)
                        {
                            sb.Append("----------------第" + (++cnt)).AppendLine("筆-------------------")
                            .Append("分公司代號=").Append(p001616_b.BrokerId).Append('\t').AppendLine("帳號=").AppendLine(p001616_b.Account)
                            .Append("Group=").AppendLine(p001616_b.Group)
                            .Append("交易員=").AppendLine(p001616_b.Trader)
                            .Append("交易所=").AppendLine(p001616_b.Exchange)
                            .Append("期(F)權(O)=").AppendLine(p001616_b.ComType)
                            .Append("商品代碼=").AppendLine(p001616_b.ComID)
                            .Append("年月=").AppendLine(p001616_b.ComYM)
                            .Append("履約價=").AppendLine(p001616_b.StrikePrice)
                            .Append("CP=").AppendLine(p001616_b.CP)
                            .Append("BS=").AppendLine(p001616_b.BS)
                            .Append("交割日期=").AppendLine(p001616_b.DeliveryDate)
                            .Append("幣別=").AppendLine(p001616_b.Currency)
                            .Append("未平倉口數=").AppendLine(p001616_b.OTQty)
                            .Append("結算價=").AppendLine(p001616_b.TrdPrice)
                            .Append("即時價=").AppendLine(p001616_b.MPrice)
                            .Append("未平倉損益=").AppendLine(p001616_b.PRTLOS)
                            .Append("成交均價=").AppendLine(p001616_b.DealPrice);
                            AddInfo(sb.ToString());
                            sb.Remove(0, sb.Length);
                        }
                    }
                    break;
                case DT.FINANCIAL_TRADER: // 分帳客戶盤中權益數查詢  
                    P001620 p001620 = (P001620)package;
                    AddInfo("1620 筆數" + p001620.Count);
                    sb = new StringBuilder();
                    if (p001620.Count > 0)
                    {
                        foreach (P001620_2 p001620_b in p001620.p001620_2)
                        {
                            sb.Append("----------------第" + (++cnt)).AppendLine("筆-------------------")
                            .Append("分公司代號=").Append(p001620_b.BrokerId).Append('\t').AppendLine(" 帳號=").AppendLine(p001620_b.Account)
                            .Append(" Group=").Append(p001620_b.Group).Append('\t').AppendLine(" 交易員=").AppendLine(p001620_b.Trader)
                            .Append(" 幣別=").AppendLine(p001620_b.Currency)
                            .Append(" 前日餘額=").AppendLine(p001620_b.LCTDAB)
                            .Append(" 手續費=").AppendLine(p001620_b.ORIGNFEE)
                            .Append("匯率=").AppendLine(p001620_b.TAXAMT)
                            .Append("期交稅=").AppendLine(p001620_b.CTAXAMT)
                            .Append("存提=").AppendLine(p001620_b.DWAMT)
                            .Append("平倉損益=").AppendLine(p001620_b.OSPRTLOS)
                            .Append("未平倉損益=").AppendLine(p001620_b.PRTLOS)
                            .Append("未沖銷買方選擇權市值=").AppendLine(p001620_b.BMKTVAL)
                            .Append("未沖銷賣方選擇權市值=").AppendLine(p001620_b.SMKTVAL)
                            .Append("委託權利金=").AppendLine(p001620_b.OPREMIUM)
                            .Append("權利金收入與支出=").AppendLine(p001620_b.TPREMIUM)
                            .Append("權益數=").AppendLine(p001620_b.EQUITY)
                            .Append("原始保證金=").AppendLine(p001620_b.IAMT)
                            .Append("維持保證金=").AppendLine(p001620_b.MAMT)
                            .Append("可用餘額=").AppendLine(p001620_b.EXCESS)
                            .Append("可動用(出金)保證金=").AppendLine(p001620_b.ORDEXCESS)
                            .Append("委託保證金=").AppendLine(p001620_b.ORDAMT)
                            .Append("到期履約損益=").AppendLine(p001620_b.ExProfit)
                            .Append("變動權利金=").AppendLine(p001620_b.Premium)
                            .Append("洗價時間=").AppendLine(p001620_b.PTime)
                            .Append("未沖銷期貨浮動損益=").AppendLine(p001620_b.FloatProfit)
                            .Append("昨日未沖銷期貨浮動損益=").AppendLine(p001620_b.LASSPRTLOS)
                            .Append("到期結算保證金=").AppendLine(p001620_b.CLOSEAMT)
                                .Append("足額原始保證金=").AppendLine(p001620_b.ORDIAMT)
                                .Append("足額維持保證金=").AppendLine(p001620_b.ORDMAMT)
                                .Append("當沖原始保證金=").AppendLine(p001620_b.DayTradeAMT)
                                .Append("多空減收保證金=").AppendLine(p001620_b.ReductionAMT)
                                .Append("當沖應補保證金=").AppendLine(p001620_b.CreditAMT)
                                .Append("本日餘額=").AppendLine(p001620_b.balance)
                                .Append("本日權利金收入=").AppendLine(p001620_b.IPremium)
                                .Append("本日權利金支出=").AppendLine(p001620_b.OPremium)
                                .Append("有價証券價值=").AppendLine(p001620_b.Securities)
                                .Append("有價証券抵繳金額=").AppendLine(p001620_b.SecuritiesOffset)
                                .Append("委託抵繳保證金=").AppendLine(p001620_b.OffsetAMT)
                                .Append("剩餘可抵繳金額=").AppendLine(p001620_b.Offset)
                                .Append("足額風險指標=").AppendLine(p001620_b.FULLMTRISK)
                                .Append("足額權益比率 =").AppendLine(p001620_b.FULLRISK)
                                .Append(" 追繳金額=").AppendLine(p001620_b.MarginCall)
                                .Append(" 賣方垂直價差市值=").AppendLine(p001620_b.SellVerticalSpread)
                                .Append(" 履約價款=").AppendLine(p001620_b.StrikePrice)
                                .Append(" 權益總值=").AppendLine(p001620_b.ActMarketValue);
                            AddInfo(sb.ToString());
                            sb.Remove(0, sb.Length);
                        }
                    }
                    break;
                case DT.FINANCIAL_TRADERN:      // 分帳客戶盤中權益數查詢 2013.6.18 新格式(ADD 3 欄位)
                    P001626 p001626 = (P001626)package;
                    AddInfo("1626 筆數" + p001626.Count);
                    sb = new StringBuilder();
                    if (p001626.Count > 0)
                    {
                        foreach (P001626_2 p001620_b in p001626.p001626_2)
                        {
                            sb.Append("----------------第" + (++cnt)).AppendLine("筆-------------------")
                             .Append("分公司代號=").Append(p001620_b.BrokerId).Append('\t').AppendLine(" 帳號=").AppendLine(p001620_b.Account)
                             .Append(" Group=").Append(p001620_b.Group).Append('\t').AppendLine(" 交易員=").AppendLine(p001620_b.Trader)
                             .Append(" 幣別=").AppendLine(p001620_b.Currency)
                             .Append(" 前日餘額=").AppendLine(p001620_b.LCTDAB)
                             .Append(" 手續費=").AppendLine(p001620_b.ORIGNFEE)
                             .Append("匯率=").AppendLine(p001620_b.TAXAMT)
                             .Append("期交稅=").AppendLine(p001620_b.CTAXAMT)
                             .Append("存提=").AppendLine(p001620_b.DWAMT)
                             .Append("平倉損益=").AppendLine(p001620_b.OSPRTLOS)
                             .Append("未平倉損益=").AppendLine(p001620_b.PRTLOS)
                             .Append("未沖銷買方選擇權市值=").AppendLine(p001620_b.BMKTVAL)
                             .Append("未沖銷賣方選擇權市值=").AppendLine(p001620_b.SMKTVAL)
                             .Append("委託權利金=").AppendLine(p001620_b.OPREMIUM)
                             .Append("權利金收入與支出=").AppendLine(p001620_b.TPREMIUM)
                             .Append("權益數=").AppendLine(p001620_b.EQUITY)
                             .Append("原始保證金=").AppendLine(p001620_b.IAMT)
                             .Append("維持保證金=").AppendLine(p001620_b.MAMT)
                             .Append("可用餘額=").AppendLine(p001620_b.EXCESS)
                             .Append("可動用(出金)保證金=").AppendLine(p001620_b.ORDEXCESS)
                             .Append("委託保證金=").AppendLine(p001620_b.ORDAMT)
                             .Append("到期履約損益=").AppendLine(p001620_b.ExProfit)
                             .Append("變動權利金=").AppendLine(p001620_b.Premium)
                             .Append("洗價時間=").AppendLine(p001620_b.PTime)
                             .Append("未沖銷期貨浮動損益=").AppendLine(p001620_b.FloatProfit)
                             .Append("昨日未沖銷期貨浮動損益=").AppendLine(p001620_b.LASSPRTLOS)
                             .Append("到期結算保證金=").AppendLine(p001620_b.CLOSEAMT)
                                 .Append("足額原始保證金=").AppendLine(p001620_b.ORDIAMT)
                                 .Append("足額維持保證金=").AppendLine(p001620_b.ORDMAMT)
                                 .Append("當沖原始保證金=").AppendLine(p001620_b.DayTradeAMT)
                                 .Append("多空減收保證金=").AppendLine(p001620_b.ReductionAMT)
                                 .Append("當沖應補保證金=").AppendLine(p001620_b.CreditAMT)
                                 .Append("本日餘額=").AppendLine(p001620_b.balance)
                                 .Append("本日權利金收入=").AppendLine(p001620_b.IPremium)
                                 .Append("本日權利金支出=").AppendLine(p001620_b.OPremium)
                                 .Append("有價証券價值=").AppendLine(p001620_b.Securities)
                                 .Append("有價証券抵繳金額=").AppendLine(p001620_b.SecuritiesOffset)
                                 .Append("委託抵繳保證金=").AppendLine(p001620_b.OffsetAMT)
                                 .Append("剩餘可抵繳金額=").AppendLine(p001620_b.Offset)
                                 .Append("足額風險指標=").AppendLine(p001620_b.FULLMTRISK)
                                 .Append("足額權益比率 =").AppendLine(p001620_b.FULLRISK)
                                 .Append(" 追繳金額=").AppendLine(p001620_b.MarginCall)
                                 .Append(" 賣方垂直價差市值=").AppendLine(p001620_b.SellVerticalSpread)
                                 .Append(" 履約價款=").AppendLine(p001620_b.StrikePrice)
                                 .Append(" 權益總值=").AppendLine(p001620_b.ActMarketValue)
                            .Append("本日期貨平倉損益淨額=").AppendLine(p001620_b.TPRTLOS)
                            .Append("超額/追繳保證金=").AppendLine(p001620_b.MarginCall1)
                            .Append("加收保證金=").AppendLine(p001620_b.AddMargin);
                            AddInfo(sb.ToString());
                            sb.Remove(0, sb.Length);
                        }
                    }
                    break;

                case DT.FINANCIAL_COVER_TRADER_Detail:
                    P001624 p001624 = (P001624)package;
                    AddInfo(p001624.ToLog());
                    break;
                #endregion
                case DT.FUT_ORDER_ACK:        //下單第二回覆
                    PT02002 pt02002 = (PT02002)package;
                    //if (pt02002.ErrorCode != 0)
                    //    msg = tfcom.GetMessageMap(pt02002.ErrorCode);

                    //AddInfo(pt02002.ToLog() + "訊息:" + tfcom.GetMessageMap(pt02002.ErrorCode));

                    break;
                case DT.FUT_ORDDER_CHANGE_ACK:      //改單第二回覆
                    PT02006 pt02006 = (PT02006)package;
                    //if (pt02006.ErrorCode != 0)
                    //    msg = " Message=" + tfcom.GetMessageMap(pt02006.ErrorCode);
                    //AddInfo("委託傳送序號=" + pt02006.RequestId
                    //    + " 委託書號=" + pt02006.OrderNo
                    //    + " 送出前台" + pt02006.FrontOffice
                    //    + " 錯誤代碼=" + pt02006.ErrorCode + Environment.NewLine
                    //    + " 訊息=" + msg);
                    //AddInfo(pt02006.ToLog() + tfcom.GetMessageMap(pt02006.ErrorCode));
                    break;
                case DT.FUT_PRODUCT_INFO:
                    //PT01800 pt01800 = (PT01800)package;
                    //sb = new StringBuilder();
                    //sb.Append("商品:").Append(pt01800.ComID).Append(" 價格小數位:").Append(pt01800.DecimalLength).Append(" 履約價格小數位:").Append(pt01800.SPPoint);
                    //AddInfo(sb.ToString());

                    //ShowPT01800(pt01800);
                    break;
                case DT.FUT_ORDER_RPT:                  //委託回報
                    try
                    {
                        PT02010 pt02010 = (PT02010)package;

                        sb = new StringBuilder();
                        sb.Append((ushort.Parse(pt02010.AfterQty.ToString().Trim()) == 0) ||
                                (string.IsNullOrEmpty(pt02010.OrderNo.ToString().Trim())) ?
                                 "刪單回報" : "委託回報"
                           ).Append(Environment.NewLine);

                        //sb.Append("委託回報").Append(Environment.NewLine);
                        //sb.Append("CNT:").Append(pt02010.CNT).Append(Environment.NewLine);
                        //sb.Append("委託時間:").Append(pt02010.ClientOrderTime).Append(Environment.NewLine);
                        sb.Append("回報時間:").Append(pt02010.ReportTime).Append(Environment.NewLine);
                        
                        sb.Append("委託書號:").Append(pt02010.OrderNo).Append(Environment.NewLine);
                        sb.Append("買賣別:").Append(pt02010.Side).Append(Environment.NewLine);
                        sb.Append("商品代號:").Append(pt02010.Symbol).Append(Environment.NewLine);
                        sb.Append("價格:").Append(pt02010.Price).Append(Environment.NewLine);
                        //sb.Append("委託方式:").Append(pt02010.TimeInForce).Append(Environment.NewLine);
                        if ((ushort.Parse(pt02010.AfterQty.ToString().Trim()) == 0) ||
                                            (string.IsNullOrEmpty(pt02010.OrderNo.ToString().Trim())))
                        {
                            sb.Append("刪單數量:").Append(pt02010.BeforeQty).Append(Environment.NewLine);
                        }
                        else
                        {
                            sb.Append("委託數量:").Append(pt02010.AfterQty).Append(Environment.NewLine);
                        }

                        //sb.Append("委託狀態:").Append(pt02010.Code).Append(Environment.NewLine);
                        //sb.Append("WebID:").Append(pt02010.WebID).Append(Environment.NewLine);

                        if (pt02010.Code != "0")
                            sb.Append("委託錯誤原因:").Append(pt02010.ErrMsg).Append(tfcom.GetMessageMap(int.Parse(pt02010.Code))).Append(Environment.NewLine);
                        //sb.Append(msgBox);

                        AddInfo(sb.ToString());

                        //AddInfo(pt02010.ToLog() + "訊息:" + tfcom.GetMessageMap(int.Parse(pt02010.Code)));



                        dataGridView1.Rows.Add(
                           ((ushort.Parse(pt02010.AfterQty.ToString().Trim()) == 0) ||
                                (string.IsNullOrEmpty(pt02010.OrderNo.ToString().Trim())) ?
                           ORDER_STATUS.ORDER_DELETED.ToString().Trim() :
                           ORDER_STATUS.ORDER_PENDING.ToString().Trim()),
                            pt02010.OrderNo.ToString().Trim(),
                            pt02010.Price.ToString().Trim(),
                            pt02010.Side.ToString().Trim(),
                            pt02010.Symbol.ToString().Trim(),
                            pt02010.TimeInForce.ToString().Trim(),
                            pt02010.AfterQty.ToString().Trim(),
                            pt02010.BeforeQty.ToString().Trim(),
                            pt02010.PositionEffect.ToString().Trim(),
                            pt02010.WebID.ToString().Trim(),
                            pt02010.CNT.ToString().Trim());



                    }
                    catch { };
                    break;
                case DT.FUT_DEAL_RPT:           //成交回報
                    try
                    {
                        PT02011 pt02011 = (PT02011)package;
                        sb = new StringBuilder();
                        sb.Append("成交回報").Append(Environment.NewLine);
                        //sb.Append("CNT:").Append(pt02011.CNT).Append(Environment.NewLine);
                        //sb.Append("交易日期:").Append(pt02011.TradeDate).Append(Environment.NewLine);
                        sb.Append("成交時間:").Append(pt02011.ReportTime).Append(Environment.NewLine);
                        sb.Append("委託書號:").Append(pt02011.OrderNo).Append(Environment.NewLine);
                        sb.Append("買賣別:").Append(pt02011.Side).Append(Environment.NewLine);
                        sb.Append("商品代號:").Append(pt02011.Symbol).Append(Environment.NewLine);
                        //sb.Append("市場:").Append(pt02011.Market).Append(Environment.NewLine);
                        sb.Append("價格:").Append(pt02011.DealPrice).Append(Environment.NewLine);
                        sb.Append("成交數量:").Append(pt02011.DealQty).Append(Environment.NewLine); ;
                        //sb.Append("已成交總量:").Append(pt02011.CumQty);
                        sb.Append("剩餘成交量:").Append(pt02011.LeaveQty).Append(Environment.NewLine);
                        //sb.Append("市場成交序號:").Append(pt02011.MarketNo);
                        //sb.Append("商品代號1:").Append(pt02011.Symbol1).Append(Environment.NewLine);
                        //sb.Append("價格1:").Append(pt02011.DealPrice1).Append(Environment.NewLine);
                        //sb.Append("成交數量1:").Append(pt02011.Qty1);
                        //sb.Append("買賣別1:").Append(pt02011.BS1).Append(Environment.NewLine);
                        //sb.Append("商品代號2:").Append(pt02011.Symbol2).Append(Environment.NewLine);
                        //sb.Append("價格2:").Append(pt02011.DealPrice2);
                        //sb.Append("成交數量2:").Append(pt02011.Qty2);
                        //sb.Append("買賣別2:").Append(pt02011.BS2).Append(Environment.NewLine);
                        AddInfo(sb.ToString());
                        //AddInfo(pt02011.ToLog());

                        dataGridView1.Rows.Add(
                            ORDER_STATUS.ORDER_CLOSED.ToString().Trim(),
                            pt02011.OrderNo.ToString().Trim(),
                            pt02011.DealPrice.ToString().Trim(),
                            pt02011.Side.ToString().Trim(),
                            pt02011.Symbol.ToString().Trim(),
                            "R",        //// TimeInForce
                           "0",
                            pt02011.DealQty.ToString().Trim(),
                            POSITION_EFFECT.PE_CLOSE.ToString().Trim(),
                            pt02011.WebID.ToString().Trim(),
                            pt02011.CNT.ToString().Trim());


                        RefreshInventoryStatus();

                    }
                    catch { };
                    break;
                case DT.FFUT_ORDER_ACK:        //下單回覆
                    PT03302 pt03302 = (PT03302)package;
                    //AddInfo(pt03302.ToLog());
                    break;
                case DT.FFUT_ORDER_RPT:                  //委託回報
                    try
                    {
                        PT03310 pt03310 = (PT03310)package;
                        //AddInfo(pt03310.ToLog());

                    }
                    catch { };
                    break;
                case DT.FFUT_DEAL_RPT:           //成交回報
                    try
                    {
                        PT03311 pt03311 = (PT03311)package;
                        //AddInfo(pt03311.ToLog());
                    }
                    catch { };
                    break;
            }
        }


        private void OnGetStatus(object sender, COM_STATUS staus, byte[] msg)
        {

            TaiFexCom com = (TaiFexCom)sender;
            // com.WriterLog("OnGetStatus step [0] COM_STATUS-> " + staus);
            if (this.InvokeRequired)
            {
                //com.WriterLog("OnGetStatus step [1] COM_STATUS-> " + staus);
                Smart.OnGetStatus_EventHandler d = new Smart.OnGetStatus_EventHandler(OnGetStatus);
                //com.WriterLog("OnGetStatus step [2] COM_STATUS-> " + staus);
                this.Invoke(d, new object[] { sender, staus, msg });
                //com.WriterLog("OnGetStatus step [3] COM_STATUS-> " + staus);
                return;
            }
            //TaiFexCom com = (TaiFexCom)sender;
            //com.WriterLog("OnGetStatusUpdateUI before COM_STATUS-> " + staus);
            OnGetStatusUpdateUI(sender, staus, msg);
            //com.WriterLog("OnGetStatusUpdateUI after COM_STATUS-> " + staus);
        }

        private void OnRecoverStatus(object sender, string Topic, RECOVER_STATUS status, uint RecoverCount)
        {
            if (this.InvokeRequired)
            {
                Smart.OnRecover_EvenHandler d = new Smart.OnRecover_EvenHandler(OnRecoverStatus);
                this.Invoke(d, new object[] { sender, Topic, status, RecoverCount });
                return;
            }

            TaiFexCom com = (TaiFexCom)sender;
            switch (status)
            {
                case RECOVER_STATUS.RS_DONE:        //回補資料結束
                    //com.WriterLog(String.Format("結束回補 Topic:[{0} 筆數:{1}]", Topic, RecoverCount));
                    if (RecoverCount == 0)
                    {
                        //AddInfo(String.Format("結束回補 Topic:[{0}]", Topic));
                    }
                    else
                    {
                        //AddInfo(String.Format("結束回補 Topic:[{0} 筆數:{1}]", Topic, RecoverCount));
                    }
                    break;
                case RECOVER_STATUS.RS_BEGIN:       //開始回補資料
                    //com.WriterLog(String.Format("開始回補 Topic:[{0}]", Topic));
                    //AddInfo(String.Format("開始回補 Topic:[{0}]", Topic));
                    break;
            }
        }




        private void OnGetStatusUpdateUI(object sender, COM_STATUS staus, byte[] msg)
        {
            TaiFexCom com = (TaiFexCom)sender;
            string smsg = null;
            switch (staus)
            {
                case COM_STATUS.LOGIN_READY:          //登入成功
                    //                    DoShowLogin();
                    AddInfo("凱基下單登入成功:" + Environment.NewLine + com.Accounts);
                    break;
                case COM_STATUS.LOGIN_FAIL:             //登入失敗
                    AddInfo(String.Format("凱基下單登入失敗:[{0}]", encoding.GetString(msg)));
                    break;
                case COM_STATUS.LOGIN_UNKNOW:       //登入狀態不明
                    AddInfo(String.Format("凱基下單登入狀態不明:[{0}]", encoding.GetString(msg)));
                    break;
                case COM_STATUS.CONNECT_READY:      //連線成功
                    smsg = "凱基下單伺服器" + tfcom.ServerHost + ":" + tfcom.ServerPort + Environment.NewLine +
                              "伺服器回應: [" + encoding.GetString(msg) + "]" + Environment.NewLine +
                              "本身為" + ((tfcom.isInternal) ? "內" : "外") + "部 IP:" + tfcom.MyIP;
                    AddInfo(smsg);
                    break;
                case COM_STATUS.CONNECT_FAIL:       //連線失敗
                    smsg = encoding.GetString(msg);
                    AddInfo("凱基下單連線失敗:" + smsg + " " + tfcom.ServerHost + ":" + tfcom.ServerPort);
                    break;
                case COM_STATUS.DISCONNECTED:       //斷線
                    //                  DoShowLogout();
                    smsg = encoding.GetString(msg);
                    AddInfo("凱基下單斷線:" + smsg);
                    break;
                case COM_STATUS.SUBSCRIBE:
                    //smsg = encoding.GetString(msg, 0, msg.Length - 1);
                    //com.WriterLog("msg.Length=" + msg.Length);
                    smsg = encoding.GetString(msg);
                    //com.WriterLog(String.Format("註冊:[{0}]", smsg));
                    //AddInfo(String.Format("註冊:[{0}]", smsg));
                    break;
                case COM_STATUS.UNSUBSCRIBE:
                    //smsg = encoding.GetString(msg, 0, msg.Length - 1);
                    smsg = encoding.GetString(msg);
                    //AddInfo(String.Format("取消註冊:[{0}]", smsg));
                    break;
                case COM_STATUS.ACK_REQUESTID:          //下單或改單第一次回覆
                    long RequestId = BitConverter.ToInt64(msg, 0);
                    byte status = msg[8];
                    //AddInfo("序號回覆: " + RequestId + " 狀態=" + ((status == 1) ? "收單" : "失敗"));
                    break;
                /*case COM_STATUS.FormLoginClick :
                    AddInfo(String.Format("Login Form Login Button Click.....: "));
                    break; */
            }

        }

        #endregion


        private void AddInfo(string msg)
        {
            if (this.msgBox.InvokeRequired)
            {
                SetAddInfoCallBack d = new SetAddInfoCallBack(AddInfo);
                this.Invoke(d, new object[] { msg });
            }
            else
            {
                string fMsg = String.Format("[{0}] {1} {2}", DateTime.Now.ToString("HH:mm:ss:ffff"), msg, Environment.NewLine);
                try
                {
                    if (msgBox.TextLength > 50000) msgBox.ResetText();
                    msgBox.AppendText(fMsg);

                }
                catch { };
            }
        }



        private void LoginBtn_MouseClick(object sender, MouseEventArgs e)
        {
            Login();
        }

        private void Login()
        {

            if (this.LoginBtn.BackColor == System.Drawing.Color.DarkGray)
            {

                AddInfo("已經登入!");
                return;

            }

           

            bAutoLogin = false;

            char area = ' ';
            //          if (rbK.Checked) area = 'K';
            //          if (rbT.Checked) area = 'T';
            string QuotedList = "TW." + SymbolBox.Text;

            ///// Quoe 
            //quoteCom = new Intelligence.QuoteCom("211.22.59.11", 8000, "API", "b6eb");
            quoteCom = new Intelligence.QuoteCom(strQuoteServerHost, 8000, "API", "b6eb"); // Host changed on Oct/01/2014
            quoteCom.OnRcvMessage += OnQuoteRcvMessage;
            quoteCom.OnGetStatus += OnQuoteGetStatus;
            quoteCom.OnRecoverStatus += OnRecoverStatusQuote;



            tfcom = new TaiFexCom(strOrderServerHost, 8000, "API", false);
            //tfcom = new TaiFexCom("10.4.99.71", 8000, "API", false);
            //tfcom = new TaiFexCom("10.4.99.71", 8000, "MPS");


            tfcom.ConnectTimeout = 5000;              //連線time out 時間 1/1000 秒
            //tfcom.ReconnectInterval = 2000;           //自動重新連線時間間隔 1/1000 秒
            //tfcom.ReconnectMaxCount = 2;             //自動重新連線.連線次數, 0,代表不自動連線

            tfcom.OnRcvMessage += OnRcvMessage;          //資料接收事件
            tfcom.OnGetStatus += OnGetStatus;               //狀態通知事件
            tfcom.OnRcvServerTime += OnRcvServerTime;   //接收主機時間
            tfcom.OnRecoverStatus += OnRecoverStatus;  //回補狀態通知




            quoteCom.SourceId = "API";
            quoteCom.Connect2Quote(strQuoteServerHost, strQuotePort, strID, strPwd, area, QuotedList);

            tfcom.ServerHost = strOrderServerHost;
            ushort.TryParse(strOrderServerPort, out tfcom.ServerPort);
            tfcom.AutoRetriveProductInfo = true;

            //tfcom.AutoRetriveForeignProductInfo = cbForeignProduct.Checked; 
            tfcom.AutoSubReport = true;
            tfcom.AutoSubReportForeign = true;

            msgBox.ResetText();


            //// Direct Login

            tfcom.LoginDirect(tfcom.ServerHost, tfcom.ServerPort, strID, strPwd, area);



            //*********Auto Reconnect  Push2 *************
            /*           
                               if (cbConn2Push.Checked)
                               {
                                   tfcom.AutoRedirectPushLogin = true;
                                   tfcom.ServerHost2 = cbHost2.Text;
                                   ushort.TryParse(cbPort2.Text, out tfcom.ServerPort2);
                                   tfcom.LoginTimeout = 8000;
                               }
             */
            //************************

            //************2012.3.27 顯示WEB on Login Form**********
            //                   tfcom.ShowWeb = cbWeb.Checked;
            //                   tfcom.URL = tbURL.Text.Trim();
            //************************************************


            // tfcom.ShowLogin();          //顯示登入視窗   ----->  this is a web-login option.

            //tfcom.Connect(tfcom.ServerHost, tfcom.ServerPort, 30);


            this.LoginBtn.BackColor = System.Drawing.Color.DarkGray;
            this.LoginBtn.ForeColor = System.Drawing.SystemColors.ControlText;


        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {

            System.IO.File.WriteAllText(strSymbolIni, SymbolBox.Text);

            Logoff();

            this.kgiTimer.Dispose();


        }

        private void Logoff()
        {

            dataGridView1.Rows.Clear();
            dataGridView1.Refresh();
            dataGridView2.Rows.Clear();
            dataGridView2.Refresh();

            futureschart.Series["Price"].Points.Clear();
            futureschart.Series["Ladder"].Points.Clear();
            bGraphicsInitialized = false;
            xx = 0;


            msgBox.ResetText();
            buylabel.Text = "";
            selllabel.Text = "";
            labelServerTime.Text = "";
            lbAskVol.Text = "";
            lbBidVol.Text = "";
            this.labelServerTime.Text = "";
            this.lbSpeed.Text = "漲跌速: -";
            this.Text = "KGI Futures";
            kgiNotifyIcon.Text = "KGI";
            
            bAutoLogin = false;
            bAutoLogoff = false;
            dPricePrev = 0;
            dPriceCurrent = 0;
            bTimerPricespeedcalled = true;

            if (bDDEConnected)
            {
                DDELogoff();
            }

            if (this.LoginBtn.BackColor != System.Drawing.SystemColors.Control)
            {
                tfcom.Logout();
                tfcom.Dispose();
                quoteCom.Logout();
                quoteCom.Dispose();
            }


            this.LoginBtn.BackColor = System.Drawing.SystemColors.Control;

        }

        private void kgiNotifyIcon_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (WindowState == FormWindowState.Minimized)
            {
                this.Show();
                this.WindowState = FormWindowState.Normal;
                this.kgiNotifyIcon.Visible = false;
            }
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Minimized)
            {
                if (string.IsNullOrEmpty(buylabel.Text))
                {
                    this.Hide();
                    this.kgiNotifyIcon.Visible = true;
                }
            }
        }


        private void SellBtn_MouseClick(object sender, MouseEventArgs e)
        {

            decimal orderPrice = 0;
            bool bBuyMarket = false;

            if (marketCB.Checked)
            {
                if (!bSlippage)
                {
                    orderPrice = 0;
                    bBuyMarket = true;
                }
                else if (!string.IsNullOrEmpty(selllabel.Text) && !string.IsNullOrEmpty(buylabel.Text))
                {
                    orderPrice = decimal.Parse(buylabel.Text.ToString().Trim());// -dPriceStep;
                }
            }
            else if (!string.IsNullOrEmpty(priceBox.Text))
            {
                orderPrice = decimal.Parse(priceBox.Text.ToString().Trim());
            }
            else
            {
                MessageBox.Show("沒有價格", "Warning");
                return;
            }

            ushort Qty = ushort.Parse(qtyBox.Text);


            ///// Confirmation of Order Placement

            string strConfirmation = "賣" + Qty.ToString() + "口" + SymbolBox.Text +
                ((marketCB.Checked) ? "市價" : ("價格" + priceBox.Text));

            if (!directCB.Checked)
            {
                if (MessageBox.Show(strConfirmation, "Confirmation",
                    MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
                {
                    return;
                }
            }

            bSell = true;
            bBuy = false;

            InstantOrder(orderPrice, Qty, bBuy, bBuyMarket);
        }

        private void BuyBtn_MouseClick(object sender, MouseEventArgs e)
        {

            decimal orderPrice = 0;
            bool bBuyMarket = false;

            if (marketCB.Checked)
            {
                if (!bSlippage)
                {
                    orderPrice = 0;
                    bBuyMarket = true;
                }
                else if (!string.IsNullOrEmpty(selllabel.Text) && !string.IsNullOrEmpty(buylabel.Text))
                {
                    orderPrice = decimal.Parse(selllabel.Text.ToString().Trim());// +dPriceStep;
                }
            }
            else if (!string.IsNullOrEmpty(priceBox.Text))
            {
                orderPrice = decimal.Parse(priceBox.Text.ToString().Trim());
            }
            else
            {
                MessageBox.Show("沒有價格", "Warning");
                return;
            }

            ushort Qty = ushort.Parse(qtyBox.Text);


            ///// Confirmation of Order Placement

            string strConfirmation = "買" + Qty.ToString() + "口" + SymbolBox.Text +
                ((marketCB.Checked) ? "市價" : ("價格" + priceBox.Text));

            if (!directCB.Checked)
            {
                if (MessageBox.Show(strConfirmation, "Confirmation",
                    MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
                {
                    return;
                }
            }

            bSell = false;
            bBuy = true;

            InstantOrder(orderPrice, Qty, bBuy, bBuyMarket);
        }

        private void InstantOrder(decimal orderPrice, ushort orderQty, bool bOrderBuySell, bool bOrderMarket)
        {

            long rtn = 0;

            MARKET_FLAG MF = MARKET_FLAG.MF_FUT;

            SIDE_FLAG SideFlag = SIDE_FLAG.SF_BUY;

            if (!bOrderBuySell)
            {
                SideFlag = SIDE_FLAG.SF_SELL;
            }
            else
            {
                SideFlag = SIDE_FLAG.SF_BUY;

            }

            PRICE_FLAG priceFlag = (bOrderMarket
                ? PRICE_FLAG.PF_MARKET : PRICE_FLAG.PF_SPECIFIED);



            TIME_IN_FORCE TIF;

            if (bOrderMarket)
            {
                TIF = TIME_IN_FORCE.TIF_IOC;
            }
            else
            {
                TIF = TIME_IN_FORCE.TIF_ROD;
            }


            POSITION_EFFECT pe;

            switch (iPositionIndex)
            {
                case 0: pe = POSITION_EFFECT.PE_OPEN; break;
                case 1: pe = POSITION_EFFECT.PE_CLOSE; break;
                case 2: pe = POSITION_EFFECT.PE_DAY_TRADE; break;
                case 3: pe = POSITION_EFFECT.PE_AUTO; break;
                default:
                    MessageBox.Show("自動倉別錯誤", "Warning");
                    return;
            }

            OFFICE_FLAG officeFlag = OFFICE_FLAG.OF_SPEEDY;


            long RequestId;

            //****注意: RequestID必須經由GetRequestID取得, 且不可重覆********

            RequestId = tfcom.GetRequestId();      //取得送單序號


            rtn = tfcom.Order(ORDER_TYPE.OT_NEW, MF, RequestId, brokerID, tbTAccount, subAccount,
                                         SymbolBox.Text, SideFlag, priceFlag, orderPrice, TIF, orderQty, pe, officeFlag);


            /*
            switch (rtn) {
                case  0:  break;
                case -1: MessageBox.Show("尚未連線"); break;
                case -2: MessageBox.Show("非允許下單帳號"); break;
                case -3: MessageBox.Show("尚未安裝簽章元件"); break;
                case -4: MessageBox.Show("非回傳帳號"); break;
                case -5: MessageBox.Show("內部IP不允許非員工戶下單"); break;
                case -6: MessageBox.Show("複式單只可使用 FOK/IOC"); break;
                case -7: MessageBox.Show("內部IP申請員工戶下單但IP不允許"); break;
                case -8: MessageBox.Show("沒有當沖權限"); break;
                case -9: MessageBox.Show("送單序號(RequestID)有誤,須使用GetRequestID取得,且不可重複!!"); break;
                case -10: MessageBox.Show("刪改單時，平台,來源別,網路單號及委託書號不可為空白!!"); break;
                case -11: MessageBox.Show("Group 無下單權限"); break;
                case -12: MessageBox.Show("非回傳 Trader "); break;
                case -13: MessageBox.Show("找不到有效憑證 "); break;
                case -14: MessageBox.Show("非MPS帳號 , 不可下單 "); break;
                default: MessageBox.Show(tfcom.GetMessageMap((ushort)rtn)); break;
            }*/
            if (rtn != 0)
                AddInfo(tfcom.GetOrderErrMsg(rtn));

            bSell = false;
            bBuy = false;
        }

        #region Quote Functions

        private void OnQuoteRcvMessage(object sender, PackageBase package)
        {
            if (package.TOPIC != null)
                if (RecoverMap.ContainsKey(package.TOPIC))
                    RecoverMap[package.TOPIC]++;

            //       int i;

            StringBuilder sb;

            switch (package.DT)
            {
                case (ushort)DT.LOGIN:
                    P001503 _p001503 = (P001503)package;
                    //        if (_p001503.Code == 0)
                    //            AddInfo("可註冊檔數：" + _p001503.Qnum);
                    break;
                case (ushort)DT.QUOTE_I020:
                    /*                    PI20020 i20020 = (PI20020)package;
                                        sb = new StringBuilder(Environment.NewLine);
                                        sb.Append("資料時間:").Append(String.Format("{0:00:00:00\\.00}", i20020.MatchTime));
                                        //                 sb.Append("    Tick序號:").Append(i20020.InfoSeq).Append(Environment.NewLine);
                                        //                 sb.Append("最後封包:").Append(i20020.LastItem).Append(Environment.NewLine);
                                        //                 sb.Append("期貨/選擇權:").Append(i20020.Market).Append("  [").Append(i20020.Symbol).Append("]").Append(Environment.NewLine);
                                        //sb.Append("價格正負號:").Append(i20020.MATCH_SIGN).Append(Environment.NewLine);
                                        sb.Append("成交 [價:").Append(i20020.Price).Append("   量:").Append(i20020.MatchQuantity).Append("]").Append(Environment.NewLine);
                                        sb.Append("累計成交  [量:").Append(i20020.MatchTotalQty).Append("  買筆數:")
                                            .Append(i20020.MatchBuyCnt).Append("  賣筆數:")
                                            .Append(i20020.MatchSellCnt).Append("]").Append(Environment.NewLine);
                                    //    AddInfo(sb.ToString());
                                        //AddInfoT("[I20]");
                                        break;
                                    case (ushort)DT.QUOTE_I020_RECOVER:   //I20 行情回補
                                        PI20020 i21020 = (PI20020)package;
                                        sb = new StringBuilder(Environment.NewLine);
                                        sb.Append("===I020 資料回補 =====").Append(Environment.NewLine);
                                        sb.Append("資料時間:").Append(String.Format("{0:00:00:00\\.00}", i21020.MatchTime));
                                        sb.Append("    Tick序號:").Append(i21020.InfoSeq).Append(Environment.NewLine);
                                        sb.Append("最後封包:").Append(i21020.LastItem).Append(Environment.NewLine);
                                        sb.Append("期貨/選擇權:").Append(i21020.Market).Append("  [").Append(i21020.Symbol).Append("]").Append(Environment.NewLine);
                                        sb.Append("成交 [價:").Append(i21020.Price).Append("   量:").Append(i21020.MatchQuantity).Append("]").Append(Environment.NewLine);
                                        sb.Append("累計成交  [量:").Append(i21020.MatchTotalQty).Append("  買筆數:")
                                            .Append(i21020.MatchBuyCnt).Append("  賣筆數:")
                                            .Append(i21020.MatchSellCnt).Append("]").Append(Environment.NewLine);
                                        //AddInfoCover(sb.ToString());
                                   //     AddInfo(sb.ToString());
                    */
                    break;

                case (ushort)DT.QUOTE_I021:
                    /*                    PI20021 i20021 = (PI20021)package;
                                        sb = new StringBuilder(Environment.NewLine);
                                        sb.Append("******DataType:").Append(i20021.DT).Append("********");
                                        sb.Append("資料時間:").Append(String.Format("{0:00:00:00\\.00}", i20021.MatchTime));
                                        sb.Append("當日最高成交價格:").Append(i20021.DayHighPrice).Append("當日最低成交價格:").Append(i20021.DayLowPrice).Append(Environment.NewLine);
                                   //     AddInfo(sb.ToString());

                                        //AddInfoT(i20021.ToLog());
                    */
                    break;
                case (ushort)DT.QUOTE_I023:
                    /*                   PI20023 i20023 = (PI20023)package;
                                       sb = new StringBuilder(Environment.NewLine);
                                       sb.Append("**********DataType:").Append(i20023.DT).Append("**********");
                                  //     AddInfo(sb.ToString());
                                       //AddInfoT(i20023.ToLog());
                    */
                    break;
                case (ushort)DT.QUOTE_I030:
                    /*                   PI20030 i20030 = (PI20030)package;
                                  //     AddInfo(i20030.ToLog());

                    */
                    break;
                case (ushort)DT.QUOTE_BASE_P08:
                    //PI20008 pi20008 = (PI20008)package;
                    //sb = new StringBuilder();
                    //sb.Append("商品:").Append(pi20008.Symbol).Append(" 價格小數位:").Append(pi20008.PriceDecimal).Append(" 履約價格小數位:").Append(pi20008.StrikePriceDecimal);
                    //AddInfo(sb.ToString());

                    //sb = new StringBuilder(Environment.NewLine);
                    //sb.Append("期貨/選擇權:").Append(pi20008.Market).Append(Environment.NewLine);
                    //sb.Append("商品代號:").Append(pi20008.Symbol).Append(Environment.NewLine);
                    //sb.Append("商品IDX:").Append(pi20008.SymbolIdx).Append(Environment.NewLine);
                    //sb.Append("第一漲停價:").Append(pi20008._RISE_LIMIT_PRICE1).Append(Environment.NewLine);
                    //sb.Append("參考價:").Append(pi20008._REFERENCE_PRICE).Append(Environment.NewLine);
                    //sb.Append("第一漲停價:").Append(pi20008._RISE_LIMIT_PRICE1).Append(Environment.NewLine);
                    //sb.Append("第二漲停價:").Append(pi20008._RISE_LIMIT_PRICE2).Append(Environment.NewLine);
                    //sb.Append("第二跌停價:").Append(pi20008._FALL_LIMIT_PRICE2).Append(Environment.NewLine);
                    //sb.Append("第三漲停價:").Append(pi20008._RISE_LIMIT_PRICE3).Append(Environment.NewLine);
                    //sb.Append("第三跌停價:").Append(pi20008._FALL_LIMIT_PRICE3).Append(Environment.NewLine);
                    //sb.Append("契約種類:").Append(pi20008._PROD_KIND).Append(Environment.NewLine);
                    //sb.Append("價格欄位小數位數:").Append(pi20008.PriceDecimal).Append(Environment.NewLine);
                    //sb.Append("商品名稱:").Append(pi20008._PROD_NAME).Append(Environment.NewLine);
                    //sb.Append("=================================");
                    //AddInfo(sb.ToString());

                    break;
                case (ushort)DT.QUOTE_CLOSE_I070:
                    /*                  PI20070 pb = (PI20070)package;
                                      sb = new StringBuilder();
                                      sb.Append("期貨/選擇權:").Append(pb.MESSAGE_KIND).Append(Environment.NewLine);
                                      sb.Append("商品代號:").Append(pb.PROD_ID).Append(Environment.NewLine);
                                      sb.Append("該期最高價:").Append(pb.TERM_HIGH_PRICE).Append(Environment.NewLine);
                                      sb.Append("該期最低價:").Append(pb.TERM_LOW_PRICE).Append(Environment.NewLine);
                                      sb.Append("該日最高價:").Append(pb.DAY_HIGH_PRICE).Append(Environment.NewLine);
                                      sb.Append("該日最低價:").Append(pb.DAY_LOW_PRICE).Append(Environment.NewLine);
                                      sb.Append("開盤價:").Append(pb.OPEN_PRICE).Append(Environment.NewLine);
                                      sb.Append("最後買價:").Append(pb.BUY_PRICE).Append(Environment.NewLine);
                                      sb.Append("最後賣價:").Append(pb.SELL_PRICE).Append(Environment.NewLine);
                                      sb.Append("收盤價:").Append(pb.CLOSE_PRICE).Append(Environment.NewLine);
                                      sb.Append("委託買進總筆數:").Append(pb.BO_COUNT_TAL).Append(Environment.NewLine);
                                      sb.Append("委託買進總口數:").Append(pb.BO_QNTY_TAL).Append(Environment.NewLine);
                                      sb.Append("委託賣出總筆數:").Append(pb.SO_COUNT_TAL).Append(Environment.NewLine);
                                      sb.Append("委託賣出總口數:").Append(pb.SO_QNTY_TAL).Append(Environment.NewLine);
                                      sb.Append("總成交筆數:").Append(pb.TOTAL_COUNT).Append(Environment.NewLine);
                                      sb.Append("總成交量:").Append(pb.TOTAL_QNTY).Append(Environment.NewLine);
                                      sb.Append("合併委託買進總筆數:").Append(pb.COMBINE_BO_COUNT_TAL).Append(Environment.NewLine);
                                      sb.Append("合併委託買進總口數:").Append(pb.COMBINE_BO_QNTY_TAL).Append(Environment.NewLine);
                                      sb.Append("合併委託賣出總筆數:").Append(pb.COMBINE_SO_COUNT_TAL).Append(Environment.NewLine);
                                      sb.Append("合併委託賣出總口數:").Append(pb.COMBINE_SO_QNTY_TAL).Append(Environment.NewLine);
                                      sb.Append("合併總成交量:").Append(pb.COMBINE_TOTAL_QNTY).Append(Environment.NewLine);
                                      sb.Append("價格欄位小數點:").Append(pb.DECIMAL_LOCATOR).Append(Environment.NewLine);
                                      sb.Append("=================");
                                  //    AddInfo(sb.ToString());
                      */
                    break;
                case (ushort)DT.QUOTE_I080:
                    PI20080 i20080 = (PI20080)package;

                    //              sb.Append("商品代號:").Append(i20080.Symbol).Append(Environment.NewLine);
                    //              for (int i = 0; i < 5; i++)

                    this.buylabel.Text = i20080.BUY_DEPTH[0].PRICE.ToString();
                    this.selllabel.Text = i20080.SELL_DEPTH[0].PRICE.ToString();

                    this.lbAskVol.Text = i20080.SELL_DEPTH[0].QUANTITY.ToString();
                    this.lbBidVol.Text = i20080.BUY_DEPTH[0].QUANTITY.ToString();

                    dPriceCurrent = i20080.BUY_DEPTH[0].PRICE;

                    if (!string.IsNullOrEmpty(buylabel.Text))
                    {
                        this.Text = "KGI " + buylabel.Text +
                                (!string.IsNullOrEmpty(mscitaiwanLbMscitaiwanprice.Text) ? " " + mscitaiwanLbMscitaiwanprice.Text : "");

                        if (kgiNotifyIcon.Visible)
                        {
                            kgiNotifyIcon.Text = buylabel.Text;
                        }

                    }



                    /////// StopBuy & StopSell


                    if ((cbStopBuy.Checked && !string.IsNullOrEmpty(tbStopBuy.Text)) ||
                        (cbStopSell.Checked && !string.IsNullOrEmpty(tbStopSell.Text)))
                    {

                        StopOrderProcess(dPriceCurrent);

                    }


                    //// Drawing chart

                    DrawPoint((double)dPriceCurrent);

                    //                  sb.Append(String.Format("五檔[{0}] 買[價:{1:N} 量:{2:N}]    賣[價:{3:N} 量:{4:N}]", i + 1, i20080.BUY_DEPTH[i].PRICE, i20080.BUY_DEPTH[i].QUANTITY, i20080.SELL_DEPTH[i].PRICE, i20080.SELL_DEPTH[i].QUANTITY)).Append(Environment.NewLine);
                    //              sb.AppendLine("衍生委託第一檔買進價格:" + i20080.FIRST_DERIVED_BUY_PRICE);
                    //              sb.AppendLine("衍生委託第一檔買進數量:" + i20080.FIRST_DERIVED_BUY_QTY);
                    //              sb.AppendLine("衍生委託第一檔賣出價格:" + i20080.FIRST_DERIVED_SELL_PRICE);
                    //              sb.AppendLine("衍生委託第一檔賣出數量" + i20080.FIRST_DERIVED_SELL_QTY);
                    //              sb.AppendLine("資料時間" + i20080.DATA_TIME);
                    //              sb.Append("==============================");
                    //              AddInfo(sb.ToString());
                    break;
                case (ushort)DT.QUOTE_I080_RECOVER:  //I080回補
                    /*                 PI20080 i21080 = (PI20080)package;
                                     sb = new StringBuilder(Environment.NewLine);
                                     sb.Append("==I080 回補資料 ===");
                                     sb.Append("商品代號:").Append(i21080.Symbol).Append(Environment.NewLine);
                                     for (i = 0; i < 5; i++)
                                         sb.Append(String.Format("五檔[{0}] 買[價:{1:N} 量:{2:N}]    賣[價:{3:N} 量:{4:N}]", i + 1, i21080.BUY_DEPTH[i].PRICE, i21080.BUY_DEPTH[i].QUANTITY, i21080.SELL_DEPTH[i].PRICE, i21080.SELL_DEPTH[i].QUANTITY)).Append(Environment.NewLine);
                                     sb.AppendLine("衍生委託第一檔買進價格:" + i21080.FIRST_DERIVED_BUY_PRICE);
                                     sb.AppendLine("衍生委託第一檔買進數量:" + i21080.FIRST_DERIVED_BUY_QTY);
                                     sb.AppendLine("衍生委託第一檔賣出價格:" + i21080.FIRST_DERIVED_SELL_PRICE);
                                     sb.AppendLine("衍生委託第一檔賣出數量" + i21080.FIRST_DERIVED_SELL_QTY);
                                     sb.AppendLine("資料時間" + i21080.DATA_TIME);
                                     //AddInfoCover(sb.ToString());
                                 //    AddInfo(sb.ToString());

                      */
                    break;
                case (ushort)DT.QUOTE_LAST_PRICE:
                    /*               PI20026 pi20026 = (PI20026)package;
                                   sb = new StringBuilder(Environment.NewLine);
                                   sb.Append("商品代號:").Append(pi20026.Symbol).Append(" 最後價格:").Append(pi20026.MatchPrice).Append(Environment.NewLine);
                                   sb.Append("當日最高成交價格:").Append(pi20026.DayHighPrice).Append(" 當日最低成交價格:").Append(pi20026.DayLowPrice);
                                   sb.Append("開盤價:").Append(pi20026.FirstMatchPrice).Append(" 開盤量:").Append(pi20026.FirstMatchQty).Append(Environment.NewLine);
                                   sb.Append("參考價:").Append(pi20026.ReferencePrice).Append(Environment.NewLine);
                              //     for (i = 0; i < 5; i++)
                              //         sb.Append(String.Format("五檔[{0}] 買[價:{1:N} 量:{2:N}]    賣[價:{3:N} 量:{4:N}]", i + 1, pi20026.BUY_DEPTH[i].PRICE, pi20026.BUY_DEPTH[i].QUANTITY, pi20026.SELL_DEPTH[i].PRICE, pi20026.SELL_DEPTH[i].QUANTITY)).Append(Environment.NewLine);
                              //     sb.Append("==========================");
                                //  AddInfo(sb.ToString());
                    */
                    break;
            }
        }


        private void OnQuoteGetStatus(object sender, COM_STATUS staus, byte[] msg)
        {
            QuoteCom com = (QuoteCom)sender;
            string smsg = null;
            switch (staus)
            {
                case COM_STATUS.LOGIN_READY:
                    AddInfo(String.Format("LOGIN_READY:[{0}]", encoding.GetString(msg)));
                    break;
                case COM_STATUS.LOGIN_FAIL:
                    AddInfo(String.Format("凱基國內期貨報價登入失敗:[{0}]", encoding.GetString(msg)));
                    break;
                case COM_STATUS.LOGIN_UNKNOW:
                    AddInfo(String.Format("凱基國內期貨報價登入原因未明:[{0}]", encoding.GetString(msg)));
                    break;
                case COM_STATUS.CONNECT_READY:
                    //quoteCom.Login(tfcom.Main_ID, tfcom.Main_PWD, tfcom.Main_CENTER);
                    smsg = "凱基國內期貨報價: [" + encoding.GetString(msg) + "] MyIP=" + quoteCom.MyIP;
                    AddInfo(smsg);
                    break;
                case COM_STATUS.CONNECT_FAIL:
                    smsg = encoding.GetString(msg);
                    AddInfo("凱基國內期貨報價連線失敗:" + smsg);
                    break;
                case COM_STATUS.DISCONNECTED:
                    smsg = encoding.GetString(msg);
                    AddInfo("凱基國內期貨報價斷線:" + smsg);
                    break;
                case COM_STATUS.SUBSCRIBE:
                    smsg = encoding.GetString(msg, 0, msg.Length - 1);
                    AddInfo(String.Format("凱基國內期貨報價註冊:[{0}]", smsg));
                    //txtQuoteList.AppendText(String.Format("SUBSCRIBE:[{0}]", smsg));  //2012.02.16 LYNN TEMPORARY ; 
                    break;
                case COM_STATUS.UNSUBSCRIBE:
                    smsg = encoding.GetString(msg, 0, msg.Length - 1);
                    AddInfo(String.Format("凱基國內期貨報價註冊取消:[{0}]", smsg));
                    break;
                case COM_STATUS.ACK_REQUESTID:
                    long RequestId = BitConverter.ToInt64(msg, 0);
                    byte status = msg[8];
                    //     AddInfo("Request Id BACK: " + RequestId + " Status=" + status);
                    break;
                case COM_STATUS.RECOVER_DATA:
                    smsg = encoding.GetString(msg, 1, msg.Length - 1);
                    if (!RecoverMap.ContainsKey(smsg))
                        RecoverMap.Add(smsg, 0);

                    if (msg[0] == 0)
                    {
                        RecoverMap[smsg] = 0;
                        //   AddInfo(String.Format("開始回補 Topic:[{0}]", smsg));
                    }

                    if (msg[0] == 1)
                    {
                        //     AddInfo(String.Format("結束回補 Topic:[{0} 筆數:{1}]", smsg, RecoverMap[smsg]));
                    }
                    break;
            }
            com.Processed();
        }






        private void OnRecoverStatusQuote(object sender, string Topic, RECOVER_STATUS status, uint RecoverCount)
        {
            if (this.InvokeRequired)
            {
                Intelligence.OnRecover_EvenHandler d = new Intelligence.OnRecover_EvenHandler(OnRecoverStatus);
                this.Invoke(d, new object[] { sender, Topic, status, RecoverCount });
                return;
            }
            QuoteCom com = (QuoteCom)sender;
            switch (status)
            {
                case RECOVER_STATUS.RS_DONE:        //回補資料結束
                    //   AddInfo(String.Format("結束回補 Topic:[{0}]{1}", Topic, RecoverCount));
                    break;
                case RECOVER_STATUS.RS_BEGIN:       //開始回補資料
                    //   AddInfo(String.Format("開始回補 Topic:[{0}]", Topic));
                    break;
            }
        }


        #endregion

        #region Charting functions

        private void DrawPoint(double dQuote)
        {


            if (!bGraphicsInitialized)
            {


                yMax = dQuote * (1.0 + dRange);
                yMin = dQuote * (1.0 - dRange);

                futureschart.ChartAreas["ChartArea1"].AxisY.Maximum = yMax;
                futureschart.ChartAreas["ChartArea1"].AxisY.Minimum = yMin;
                futureschart.ChartAreas["ChartArea1"].AxisY.LabelStyle.Format = "F0";
                futureschart.ChartAreas["ChartArea1"].AxisY.Interval = yBoundary * 10.0;

                futureschart.ChartAreas["ChartArea1"].AxisX.Minimum = 0;
                futureschart.ChartAreas["ChartArea1"].AxisX.Interval = 1;
                futureschart.ChartAreas["ChartArea1"].AxisX.IsStartedFromZero = true;


                futureschart.Series["Price"].Color = Color.Blue;
                futureschart.Series["Price"].BorderWidth = 2;

                futureschart.Series["Ladder"].Color = Color.Aqua;
                futureschart.Series["Ladder"].BorderWidth = 2;

               

                yBoxMax = dQuote + yBoundary;
                yBoxMin = dQuote - yBoundary;

                dLadder = dQuote - dLadderSize;
                dLadderPrev = dLadder;

                futureschart.Series["Price"].Points.AddY(dQuote);
                futureschart.Series["Ladder"].Points.AddY(dLadder);


                bGraphicsInitialized = true;
            }

            if (dQuote > yMax * (1.0 - dRange * 0.2))
            {
                yMax = yMax * (1.0 + dRange * 0.2);

                futureschart.ChartAreas["ChartArea1"].AxisY.Maximum = yMax;
            }

            if (dQuote < yMin * (1.0 + dRange * 0.2))
            {
                yMin = yMin * (1.0 - dRange * 0.2);

                futureschart.ChartAreas["ChartArea1"].AxisY.Minimum = yMin;
            }

            if (xx > 250)
            {
                futureschart.ChartAreas["ChartArea1"].AxisX.Minimum = Convert.ToDouble(xx - 250);
            }


            if (dQuote > yBoxMax || dQuote < yBoxMin)
            {
                xx++;

                yBoxMax = dQuote + yBoundary;
                yBoxMin = dQuote - yBoundary;

                ////// Ladder Algorithm

                if (dQuote < dLadder && bLadderBottom)
                {
                    bLadderBottom = false;

                }

                if (dQuote > dLadder && !bLadderBottom)
                {
                    bLadderBottom = true;

                }

                if (bLadderBottom)
                {
                    dLadder = dQuote - dLadderSize;
                    if (dLadder <= dLadderPrev)
                    {
                        dLadder = dLadderPrev;
                    }
                }
                else
                {
                    dLadder = dQuote + dLadderSize;
                    if (dLadder >= dLadderPrev)
                    {
                        dLadder = dLadderPrev;
                    }
                }



                if (bLadderBottom)
                {
                    futureschart.Series["Price"].Points.AddXY(xx, dQuote);
                    futureschart.Series["Ladder"].Points.AddXY(xx, dLadder);
                }
                else
                {
                    futureschart.Series["Price"].Points.AddXY(xx, dQuote);
                    futureschart.Series["Ladder"].Points.AddXY(xx, dLadder);
                }


                dLadderPrev = dLadder;

                DataSet dsSeries = futureschart.DataManipulator.ExportSeriesValues();

              

               

            }


        }

       
        #endregion



        private void cbInvisible_Click(object sender, EventArgs e)
        {

            if (cbInvisible.Checked)
            {

                futureschart.Visible = false;
                tabControlKGITwn.Visible = false;
                this.Width = 320;
                this.CenterToScreen();
            }

            if (!cbInvisible.Checked)
            {

                futureschart.Visible = true;
                tabControlKGITwn.Visible = true;
                this.Width = 1020;
                this.CenterToScreen();
            }


        }

        /// Stop Order Process

        private void cbStopBuyChanged(object sender, EventArgs e)
        {

            decimal priceStopBuy;

            if (cbStopBuy.Checked)
            {

                if (!string.IsNullOrEmpty(tbStopBuy.Text))
                {
                    priceStopBuy = decimal.Parse(tbStopBuy.Text);

                    if (!string.IsNullOrEmpty(buylabel.Text) && (priceStopBuy < decimal.Parse(buylabel.Text)))
                    {
                        cbStopBuy.Checked = false;
                        tbStopBuy.Text = "";
                        MessageBox.Show("追買價格必需大於現價", "Warning");
                        return;
                    }
                }

                else
                {
                    cbStopBuy.Checked = false;
                    MessageBox.Show("沒有價格", "Warning");
                    return;
                }
            }

            else   /// checkbox unchecked
            {
                tbStopBuy.Text = "";
            }

            return;

        }

        private void cbStopSellChanged(object sender, EventArgs e)
        {
            decimal priceStopSell;

            if (cbStopSell.Checked)
            {

                if (!string.IsNullOrEmpty(tbStopSell.Text))
                {
                    priceStopSell = decimal.Parse(tbStopSell.Text);

                    if (!string.IsNullOrEmpty(buylabel.Text) && (priceStopSell > decimal.Parse(buylabel.Text)))
                    {
                        cbStopSell.Checked = false;
                        tbStopSell.Text = "";
                        MessageBox.Show("追賣價格必需小於現價", "Warning");
                        return;
                    }
                }

                else
                {
                    cbStopSell.Checked = false;
                    MessageBox.Show("沒有價格", "Warning");
                    return;
                }
            }

            else   /// checkbox unchecked
            {
                tbStopSell.Text = "";
            }

            return;


        }

        private void marketCB_CheckedChanged(object sender, EventArgs e)
        {
            if (marketCB.Checked) { 
                priceBox.BackColor = System.Drawing.SystemColors.ScrollBar; 
                priceBox.Text = ""; }
            if (!marketCB.Checked) { 
                priceBox.BackColor = System.Drawing.SystemColors.Window; }
        }

        private void StopOrderProcess(decimal dCurPrice)
        {

            decimal stopPrice;
            string strConfirmation;

            if (cbStopBuy.Checked)
            {
                stopPrice = decimal.Parse(tbStopBuy.Text);

                if (stopPrice <= dCurPrice)
                {
                    cbStopBuy.Checked = false;
                    tbStopBuy.Text = "";

                    stopPrice += dPriceStep;

                    /*
                    strConfirmation = "追買" + qtyBox.Text + "口" + symbolID + "於" + stopPrice.ToString();

                    if (!directCB.Checked)
                    {
                        if (MessageBox.Show(strConfirmation, "Confirmation",
                            MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
                        {
                            return;
                        }
                    }*/

                    InstantOrder(stopPrice, ushort.Parse(qtyBox.Text), true, false);
                    // true - BUY MODE, false - non-market order limited with slippage


                    if (bSound)
                    {
                        playSignal.Play();
                    }

                }
            }


            if (cbStopSell.Checked)
            {
                stopPrice = decimal.Parse(tbStopSell.Text);

                if (stopPrice > dCurPrice)
                {
                    cbStopSell.Checked = false;
                    tbStopSell.Text = "";

                    stopPrice -= dPriceStep;

                    /*
                    strConfirmation = "追賣" + qtyBox.Text + "口" + symbolID + "於" + stopPrice.ToString();


                    if (!directCB.Checked)
                    {
                        if (MessageBox.Show(strConfirmation, "Confirmation",
                            MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
                        {
                            return;
                        }
                    }
                     */

                    InstantOrder(stopPrice, ushort.Parse(qtyBox.Text), false, false);
                    // false - SELL MODE, false - non-market order limited with slippage


                    if (bSound)
                    {
                        playSignal.Play();
                    }
                }
            }


            return;


        }



        private void btnDelete_Click(object sender, EventArgs e)
        {
            int i = 0, j = 0;
            long RequestId = 0, rtn = 0;
            SIDE_FLAG sfDelete = SIDE_FLAG.SF_BUY;
            TIME_IN_FORCE TIF = TIME_IN_FORCE.TIF_ROD;

            bool bPending = false;
            int dgvRowCount1 = dataGridView1.Rows.Count - 1;
            int dgvRowCount2 = dataGridView2.Rows.Count - 1;
            string strOrderID = "";


            if (MessageBox.Show("全部刪單?", "Confirmation",
                   MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
            {
                return;
            }


            //// Clean up dataGridView2

            for (i = 0; i < dgvRowCount2; i++)
            {
                dataGridView2.Rows.RemoveAt(0);

            }

            //// Copy dataGridView1 to dataGridView2

            dataGridView2.Rows.Add(dgvRowCount1);

            for (i = 0; i < dataGridView1.ColumnCount; i++)
            {

                for (j = 0; j < dgvRowCount1; j++)
                {

                    dataGridView2.Rows[j].Cells[i].Value = dataGridView1.Rows[j].Cells[i].Value.ToString().Trim();

                }
            }


            ////    Retrieve the real pending orders

            dgvRowCount2 = dataGridView2.Rows.Count - 1;

            for (i = 0; i < dgvRowCount2; i++)
            {

                if (string.Equals(ORDER_STATUS.ORDER_PENDING.ToString().Trim(), dataGridView2.Rows[i].Cells[0].Value.ToString().Trim()))
                {

                    strOrderID = dataGridView2.Rows[i].Cells[1].Value.ToString().Trim();

                    bPending = true;

                    for (j = i + 1; j < dgvRowCount2; j++)
                    {
                        if (string.Equals(strOrderID, dataGridView2.Rows[j].Cells[1].Value.ToString().Trim())
                                &&
                            !string.Equals(ORDER_STATUS.ORDER_PENDING.ToString().Trim(),
                                    dataGridView2.Rows[j].Cells[0].Value.ToString().Trim()))
                        {
                            bPending = false;
                        }
                        //      AddInfo(i.ToString() + " " + j.ToString()+" "+ strOrderID+"-"+ ( bPending ? "Pending" : "Not Pending"));
                    }   ///// j
                }


                //// delete codes


                if (bPending)
                {

                    if (string.Equals(dataGridView2.Rows[i].Cells[3].Value.ToString().Trim(), "S"))
                        sfDelete = SIDE_FLAG.SF_SELL;

                    if (string.Equals(dataGridView2.Rows[i].Cells[3].Value.ToString().Trim(), "B"))
                        sfDelete = SIDE_FLAG.SF_BUY;


                    switch (dataGridView2.Rows[i].Cells[5].Value.ToString().Trim())
                    {
                        case "R": TIF = TIME_IN_FORCE.TIF_ROD; break;
                        case "F": TIF = TIME_IN_FORCE.TIF_FOK; break;
                        case "I": TIF = TIME_IN_FORCE.TIF_IOC; break;
                        default:
                            MessageBox.Show("ROD/FOK/IOC 項目錯誤");
                            return;
                    }



                    RequestId = tfcom.GetRequestId();      //取得送單序號


                    rtn = tfcom.Order(ORDER_TYPE.OT_CANCEL,
                                                 MARKET_FLAG.MF_FUT,
                                                 RequestId,
                                                 brokerID,
                                                 tbTAccount,
                                                 subAccount,
                                                 dataGridView2.Rows[i].Cells[4].Value.ToString().Trim(), /// Symbol
                                                 sfDelete,
                                                 PRICE_FLAG.PF_MARKET,
                                                 decimal.Parse(dataGridView2.Rows[i].Cells[2].Value.ToString().Trim()), /// Price
                                                 TIF,
                                                 ushort.Parse(dataGridView2.Rows[i].Cells[6].Value.ToString().Trim()), /// Qty
                                                 POSITION_EFFECT.PE_OPEN,
                                                 OFFICE_FLAG.OF_SPEEDY,
                                                 dataGridView2.Rows[i].Cells[9].Value.ToString().Trim(),  /// Webid
                                                 dataGridView2.Rows[i].Cells[10].Value.ToString().Trim(),  /// CNT
                                                 dataGridView2.Rows[i].Cells[1].Value.ToString().Trim()    ///OrderNo
                                                 );


                    if (rtn != 0)
                        MessageBox.Show(tfcom.GetOrderErrMsg(rtn));
                }
                /////////

                bPending = false;

            }   //// i
            return;
        }



        private void btnReset_Click(object sender, EventArgs e)
        {
            qtyBox.Text = "1";
            priceBox.Text = "";
        }

        private void btnPriceUp_Click(object sender, EventArgs e)
        {

            if (marketCB.Checked) return;

            decimal dPriceUp;

            if (string.IsNullOrEmpty(priceBox.Text))
            {
                priceBox.Text = selllabel.Text;
                return;
            }
            else
            {
                dPriceUp = decimal.Parse(priceBox.Text) + dPriceStep;
                priceBox.Text = dPriceUp.ToString().Trim();
            }
        }

        private void btnPriceDown_Click(object sender, EventArgs e)
        {

            if (marketCB.Checked) return;

            decimal dPriceDown;

            if (string.IsNullOrEmpty(priceBox.Text))
            {
                priceBox.Text = buylabel.Text;
                return;
            }
            else
            {
                dPriceDown = decimal.Parse(priceBox.Text) - dPriceStep;
                priceBox.Text = dPriceDown.ToString().Trim();
            }
        }

        private void btnVolDn_Click(object sender, EventArgs e)
        {

            int usVolDn;


            usVolDn = int.Parse(qtyBox.Text) - 1;

            if (usVolDn < 1)
            {
                usVolDn = 1;
            }
            qtyBox.Text = usVolDn.ToString().Trim();

        }

        private void btnVolUp_Click(object sender, EventArgs e)
        {
            int usVolUp;

            usVolUp = int.Parse(qtyBox.Text) + 1;

            if (usVolUp > iQtyLimit)
            {
                usVolUp = iQtyLimit;
            }
            qtyBox.Text = usVolUp.ToString().Trim();
        }

        private void qtyBox_TextChanged(object sender, EventArgs e)
        {
            if (int.Parse(qtyBox.Text) > iQtyLimit)
            {
                MessageBox.Show("口數超過限額" + iQtyLimit.ToString() + "口", "Warning");
                qtyBox.Text = "1";
                return;
            }
        }

        private void RefreshInventoryStatus()
        {
            int i = 0, j = 0;
            int dgvRowCount1 = dataGridView1.Rows.Count - 1;
            int dgvRowCount2 = dataGridView2.Rows.Count - 1;

            int iInventory = 0;
            StringBuilder sb = new StringBuilder();
            decimal dInventoryCost = 0;
            decimal dInventoryAverageCost = 0;
            bool bBuySell = false;

            //// Clean up dataGridView2

            for (i = 0; i < dgvRowCount2; i++)
            {
                dataGridView2.Rows.RemoveAt(0);

            }

            //// Copy dataGridView1 to dataGridView2

            dataGridView2.Rows.Add(dgvRowCount1);

            for (i = 0; i < dataGridView1.ColumnCount; i++)
            {

                for (j = 0; j < dgvRowCount1; j++)
                {

                    dataGridView2.Rows[j].Cells[i].Value = dataGridView1.Rows[j].Cells[i].Value.ToString().Trim();

                }
            }

            ////    Retrieve the closed orders

            dgvRowCount2 = dataGridView2.Rows.Count - 1;

            for (i = 0; i < dgvRowCount2; i++)
            {

                if (string.Equals(ORDER_STATUS.ORDER_CLOSED.ToString().Trim(),
                    dataGridView2.Rows[i].Cells[0].Value.ToString().Trim()))
                {

                    /// Cell 3: buy or sell status

                    if (string.Equals(dataGridView2.Rows[i].Cells[3].Value.ToString().Trim(), "S"))
                    {
                        bBuySell = false;
                    }
                    else if (string.Equals(dataGridView2.Rows[i].Cells[3].Value.ToString().Trim(), "B"))
                    {
                        bBuySell = true;
                    }
                    else
                    {
                        AddInfo("持倉買賣別存在問題.");
                        return;
                    }

                    // Cell 7 - BeforeQty; Cell 2: Deal Price /// DealQuantity ---> Cell 7 BeforeQty written at DataGrid1

                    iInventory += ushort.Parse(dataGridView2.Rows[i].Cells[7].Value.ToString().Trim()) *
                        (bBuySell ? 1 : -1);
                    dInventoryCost += ushort.Parse(dataGridView2.Rows[i].Cells[7].Value.ToString().Trim()) *
                        (bBuySell ? 1 : -1) * decimal.Parse(dataGridView2.Rows[i].Cells[2].Value.ToString().Trim());

                    if (iInventory == 0)
                    {
                        dInventoryCost = 0;
                    }
                }
            }

            sb.Append("持倉更新:").Append(Environment.NewLine);

            if (iInventory == 0)
            {
                sb.Append(SymbolBox.Text).Append("持倉口數: ").
                   Append(iInventory.ToString()).Append(Environment.NewLine);
                AddInfo(sb.ToString());
            }
            else
            {
                dInventoryAverageCost = dInventoryCost / Math.Abs(iInventory);

                sb.Append(SymbolBox.Text).Append("持倉口數: ").
                    Append(iInventory.ToString()).Append(Environment.NewLine);
                sb.Append(SymbolBox.Text).Append("持倉成本: ").
                    Append(dInventoryAverageCost.ToString("F2")).Append(Environment.NewLine);

                AddInfo(sb.ToString());
            }


        }


      


        #region MSCI_TAIWAN
        
        private void mscitaiwanCbEnabled_CheckedChanged(object sender, EventArgs e)
        {

            if (mscitaiwanCbEnabled.Checked)
            {
                if (!bDDEConnected)
                {

                    dcBid = new DdeClient(strMT4Service, strBIDTopic);

                    try
                    {
                        dcBid.Connect();
                        dcBid.StartAdvise(strMsciSymbol, 1, true, 60000);

                    }
                    catch (Exception thrown)
                    {
                        AddInfo("無法連接 MT4 DDE Server");
                        bDDEConnected = false;
                        return;
                    }



                    dcBid.Advise += OnDDEBidAdvise;
                    AddInfo("MT4 DDE Server 摩台指報價連接成功");
                    bDDEConnected = true;
                    this.mscitaiwanLbSpread.Visible = true;
                    this.mscitaiwanLbTaiex.Visible = true;
                    this.mscitaiwanLbMSCITaiwan.Visible = true;
                    mscitaiwanLbMscitaiwanprice.Text = "";
                    msciTaiwanLbSpreadPrice.Text = "";
                    mscitaiwanLbTaiexprice.Text = "";
                }

            }

            else
            
            {
                if (bDDEConnected)
                {
                    DDELogoff();
                }
            }


        }

        private void DDELogoff()
        {
            dcBid.StopAdvise(strMsciSymbol, 60000);
            dcBid.Dispose();
            bDDEConnected = false;
            AddInfo("退出 MT4 DDE Server 連接");
            mscitaiwanLbMscitaiwanprice.Text = "";
            msciTaiwanLbSpreadPrice.Text = "";
            mscitaiwanLbTaiexprice.Text = "";
            this.mscitaiwanLbSpread.Visible = false;
            this.mscitaiwanLbTaiex.Visible = false;
            this.mscitaiwanLbMSCITaiwan.Visible = false;
            mscitaiwanCbEnabled.Checked = false;
        }

        private void OnDDEBidAdvise(object sender, DdeAdviseEventArgs args)
        {

            if (args.Item == strMsciSymbol)
            {
                
                mscitaiwanLbMscitaiwanprice.Text = args.Text;

                if (!string.IsNullOrEmpty(mscitaiwanLbTaiexprice.Text))
                {
                    msciTaiwanLbSpreadPrice.Text =
                            strSpreadConversion(
                                mscitaiwanLbMscitaiwanprice.Text,
                                mscitaiwanLbTaiexprice.Text,
                                30.0 * 100.0 / 1000.0, 2.0 * 50.0 / 1000.0, "F2");

                    // MSCI_Taiwan: US=NT30; 1 point = US100
                    // Taiex: 2 Mini-Taiex contract ~= 1 MSCI; 1 point = NT50
                }
            }

           
        }


        private void buylabel_TextChanged(object sender, EventArgs e)
        {

            if (string.IsNullOrEmpty(buylabel.Text)) return;

            if (bDDEConnected)
            {

                mscitaiwanLbTaiexprice.Text = buylabel.Text;


                if (!string.IsNullOrEmpty(mscitaiwanLbMscitaiwanprice.Text))
                {
                    msciTaiwanLbSpreadPrice.Text = 
                            strSpreadConversion(
                                mscitaiwanLbMscitaiwanprice.Text, 
                                mscitaiwanLbTaiexprice.Text,
                                30.0 * 100.0 / 1000.0, 2.0 * 50.0 / 1000.0, "F2");
                    
                    // MSCI_Taiwan: US=NT30; 1 point = US100
                    // Taiex: 2 Mini-Taiex contract ~= 1 MSCI; 1 point = NT50
                }

            }

        }


        private string strSpreadConversion(string strSymbolA, string strSymbolB,
                                           double dFactorA, double dFactorB, string strFormat)
        {

            double dSymbolA = double.Parse(strSymbolA);
            double dSymbolB = double.Parse(strSymbolB);
            double dSpread =
                dSymbolA * dFactorA - 
                dSymbolB * dFactorB; 
               

            return(dSpread.ToString(strFormat));


        }


        #endregion

       

       

      

       










    }


}

